package it.polito.elite.ontology;

import it.polito.elite.visualization.ontoviewer.enums.NodeTypesEnum;

import java.util.HashSet;

public abstract class NodeTypeAbstraction {
	protected NodeTypesEnum type;
	
	protected boolean isLogicalClass=false, isRestriction=false, isEnumeration=false;
	
	public boolean isEnumeration() {
		return isEnumeration;
	}

	public boolean isLogicalClass() {
		return isLogicalClass;
	}

	public boolean isRestriction() {
		return isRestriction;
	}

	public NodeTypesEnum getType() {
		return type;
	}
	
	/**
	 * Represents the node stored in the internal model of the subclass 
	 * with a textual expression, an example format could be:
	 *  UnionOf( ConceptA, ComplementOf(ConceptB),..) - OneOf(InstA, InstB)
	 *  with the pourpose to be displayed on Screen
	 */
	abstract public String toString();
	
	/**
	 * Retrieves the set of NodeTypeAbstraction forming the union logical class or
	 * the intersection. Returns null otherwise
	 * @return a set of NodeTypeAbstraction, if type is Union or Intersection, null otherwise
	 */
	abstract public HashSet<NodeTypeAbstraction> getLogicSet();
	/**
	 * Retrieves the set of NodeTypeAbstraction in a OneOf enumeration
	 * @return a set of OneOf, if type is Enumeration, null otherwise
	 */
	abstract public HashSet<NodeTypeAbstraction> getEnumerationSet();
	/**
	 * Retrieves the NodeTypeAbstraction filler of the restrictions, if it is a 
	 * HAS_VALUE or Qualified Cardinality or a SOME / ALL values TO restrictions
	 * @return a  NodeTypeAbstraction, if type is correct type, null otherwise
	 */
	abstract public NodeTypeAbstraction getRestrictionFiller();
	/**
	 * Retrieves the URI of the property restriction is on
	 * @return a set of NodeTypeAbstraction, if type is Union or Intersection, null otherwise
	 */
	abstract public String getRestrictionPropertyURI();
	/**
	 * Retrieves the cardinality of the restriction if type is Cardinality (base, min o max)
	 * @return a int value > 0 or -1 if not a cardinality restriction type
	 */
	abstract public int getCardinality();
	
	/**
	 * Base Comparison
	 * @param t a NodeTypeAbstraction
	 * @return true if they are equals, false otherwhise. It compares the two textual representations
	 */
	public boolean equals(NodeTypeAbstraction t){
		String a=this.toString();
		String b=t.toString();
		return a.equals(b);
	}
	
	public int hashCode(){
		return this.toString().hashCode();
	}
	
}
