package it.polito.elite.ontosphere3dplugin;

/**
 * This class implements a view showing a class tree control with classes and instances of a given ontology
 * @author Alessio Bosca
 */
import it.polito.elite.visualization.ontoviewer.models.ConceptData;
import it.polito.elite.visualization.ontoviewer.models.KnowledgeBaseModel;
import it.polito.elite.visualization.ontoviewer.scenes3D.SceneFocusConcept3;
import it.polito.elite.visualization.ontoviewer.scenes3D.SceneFocusFacts5;
import it.polito.elite.visualization.ontoviewer.userInterfaces.Pane3DScene;

import java.io.IOException;
import java.net.URL;
import java.util.Iterator;
import java.util.Set;
import java.util.Vector;

import org.eclipse.core.runtime.FileLocator;
import org.eclipse.core.runtime.Platform;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Tree;
import org.eclipse.swt.widgets.TreeItem;
import org.eclipse.ui.part.ViewPart;
import org.osgi.framework.Bundle;

public class ClassTreeView extends ViewPart {

public static final String ID_VIEW="it.polito.elite.ontosphere3DPlugin.ClassTreeView";

private Tree tree;
private Composite shell;
private Vector<TreeItem>allItem;
private Image imageClass,imageInstance;
private String itemText;	
public static ClassTreeView theInstance = null;
	
	public ClassTreeView() {}
    
	/**
	 * the method creates the actual graphical interface. Automatically called by the eclipse framework.
	 * not to be directly used
	 */
	public void createPartControl(Composite parent)
    {	
		
		try{
			shell=parent;
    		tree=new Tree(parent,SWT.BORDER);
    		this.treeListener();
    		this.createTree();
    		ClassTreeView.theInstance = this;
    	}catch (Exception e){
    		e.printStackTrace();
    	}

    }
	
	/**
	 * Loads I
	 * @throws Exception
	 */
private void createTree () throws IOException{
		
		Bundle bundle = Platform.getBundle("it.polito.elite.ontosphere3dplugin");
		URL url = this.getClass().getClassLoader().getResource("resources/class.gif");
		//Path path = new Path("icons/class.gif");
		//URL url = FileLocator.find(bundle, path, null);
		String filePath = FileLocator.toFileURL(url).getPath();
		imageClass=new Image(shell.getDisplay(),filePath);
		
		 //path = new Path("icons/instance.gif");
		 //url = FileLocator.find(bundle, path, null);
		url = this.getClass().getClassLoader().getResource("resources/instance.gif");
		filePath = FileLocator.toFileURL(url).getPath();
		imageInstance=new Image(shell.getDisplay(),filePath);
		
		Pane3DScene my3DPanel = OntoSelectorView.getViewerPanel();
		if(my3DPanel != null ){
			KnowledgeBaseModel kbModel = my3DPanel.model;
			if(kbModel != null)
				this.updateView(kbModel);
		}
	}
	
	public void updateView(KnowledgeBaseModel kbModel){
		if (this.tree != null && this.tree.getItemCount() > 0){
			this.tree.removeAll();
		}
		
		for(String classname: kbModel.getRoots()){
			System.out.println("- "+classname);
			TreeItem item=new TreeItem(tree,SWT.NULL);
			allItem=new Vector<TreeItem>();
			allItem.add(item);
			navigate(classname,item, kbModel);
		}

		this.tree.redraw();
	}
	
	private void treeListener()
	{	
		tree.addSelectionListener(new SelectionAdapter(){
			public void widgetSelected(SelectionEvent e)
			{
				TreeItem[] t = tree.getSelection();
				itemText=t[0].getText();
				showElement(itemText);
			}	
		});
	}
	
	public void showElement(String elementLocalName){
		Pane3DScene panel3D=OntoSelectorView.getViewerPanel();
		if ( panel3D.model.getConceptData(itemText) != null){
			SceneFocusConcept3 newScene = new SceneFocusConcept3(panel3D, itemText);
			OntoSelectorView.getViewerPanel().changeScene(newScene);
		}
		else{
			String concept = panel3D.model.getInstanceData(elementLocalName).getPertainingConceptURI(); 
			SceneFocusFacts5 newScene = new SceneFocusFacts5(panel3D, concept, itemText);
			panel3D.changeScene(newScene);
		}
	}

	private void navigate(String cl,TreeItem item,KnowledgeBaseModel model)
	{
		ConceptData data = model.getConceptData(cl);
		Set<String>subClasses=data.getChildren();
		TreeItem item2=null,item3=null;
		item.setText(cl);
		item.setImage(imageClass);
		
		if(data.hasInstances())
		{
			Set<String> istances=data.getInstancesURI();
			Iterator it=istances.iterator();
			Object sel;
			while (it.hasNext())
			{
				sel=it.next();
				item3=new TreeItem(item,SWT.NULL);
				allItem.add(item3);
				item3.setForeground(item3.getDisplay().getSystemColor(SWT.COLOR_RED));
				item3.setText(sel.toString());
				item3.setImage(imageInstance);
			}
		}
	
		if(!subClasses.isEmpty())
		{
			Iterator it=subClasses.iterator();
			Object selected;
			while (it.hasNext())
			{
				selected=it.next();
				item2=new TreeItem(item,SWT.NULL);
				allItem.add(item2);
				navigate(selected.toString(),item2,model);
			}
		}
	}
	
	
	@Override
	public void setFocus() {	}

	public String getSelectedItem() {
		return itemText;
	}
	
}
