package it.polito.elite.ontosphere3dplugin;

import it.polito.elite.visualization.ontoviewer.userInterfaces.Pane3DScene;

import java.net.URI;
import java.net.URISyntaxException;

import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.part.ViewPart;

import edu.stanford.smi.protege.model.KnowledgeBase;
import edu.stanford.smi.protegex.owl.ProtegeOWL;
import edu.stanford.smi.protegex.owl.jena.JenaOWLModel;

public class OntoSelectorView extends ViewPart {

	public static final String ID_VIEW = "it.polito.elite.ontosphere3DPlugin.OntoSelector";
	private Label lab;

	static private Pane3DScene panel3D=null;

	public void createPartControl(final Composite parent) {
		
		GridLayout gridLayout = new GridLayout(2, false);
		gridLayout.horizontalSpacing = 10;
		parent.setLayout(gridLayout);

		final Text text = new Text(parent, org.eclipse.swt.SWT.SINGLE
				| org.eclipse.swt.SWT.LEFT | org.eclipse.swt.SWT.BORDER);
		GridData gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.widthHint = 200;
		gd.horizontalSpan = 2;
		text.setText("http://www.co-ode.org/ontologies/pizza/2005/05/16/pizza.owl");//
		text.setToolTipText("Insert an ontology uri (OWL  files only)");
		text.setLayoutData(gd);

		Button bt = new Button(parent, org.eclipse.swt.SWT.PUSH);
		bt.setText(" Load Ontology ");
		bt.setLayoutData(new GridData());

		lab = new Label(parent, org.eclipse.swt.SWT.BORDER);
		lab.setText(" NO ONTOLOGY LOADED ");
		lab.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));

		bt.addSelectionListener(new SelectionListener() {

			public void widgetSelected(SelectionEvent event) {
				try {
					String str = text.getText();
					URI ontologyURI = new URI(str);
					updateKB(ontologyURI);
				} catch (URISyntaxException e) {
					lab.setText(" CHECK URI SYNTAX! ");
				}

			}

			public void widgetDefaultSelected(SelectionEvent event) {
			}

		});

	}

	public void updateKB(URI ontologyURI) {
		JenaOWLModel model;
		try {
			model = ProtegeOWL
					.createJenaOWLModelFromURI(ontologyURI.toString());
			KnowledgeBase kb = model.getProject().getKnowledgeBase();
			lab.setText(" ONTOLOGY LOADED ");
			OntoSelectorView.panel3D = new Pane3DScene(kb);
			
			OntoSphereView myMainViewInstance = OntoSphereView.theInstance;
			if (myMainViewInstance != null) 
				myMainViewInstance.updateView(OntoSelectorView.panel3D);
			
			ClassTreeView myTreeViewInstance = ClassTreeView.theInstance;
			if (myTreeViewInstance != null)
					myTreeViewInstance.updateView(OntoSelectorView.panel3D.model);

			SubjectAreaView mySubjectAreaInstance = SubjectAreaView.theInstance;
			if (mySubjectAreaInstance != null)
				mySubjectAreaInstance.updateView(OntoSelectorView.panel3D);

			PropertyView myPropertyInstance = PropertyView.theInstance;
			if (myPropertyInstance != null)
					myPropertyInstance.updateView(OntoSelectorView.panel3D.model);
		} catch (Exception e) {
			lab.setText(" ERROR LOADING THE ONTOLOGY! ");
			e.printStackTrace();
		}
		
	}

	public void setFocus() {		}
	
	static public Pane3DScene getViewerPanel(){
		return OntoSelectorView.panel3D;
	}
	
}
