package it.polito.elite.ontosphere3dplugin;

import it.polito.elite.ontology.NodeTypeAbstraction;
import it.polito.elite.visualization.ontoviewer.models.KnowledgeBaseModel;
import it.polito.elite.visualization.ontoviewer.models.RelationType;
import it.polito.elite.visualization.ontoviewer.userInterfaces.Pane3DScene;

import java.io.IOException;
import java.net.URL;
import java.util.Hashtable;

import org.eclipse.core.runtime.FileLocator;
import org.eclipse.core.runtime.Platform;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.TableItem;
import org.eclipse.ui.part.ViewPart;
import org.osgi.framework.Bundle;

public class PropertyView extends ViewPart {
	private Table table = null;

	private Image imageOK, imageNOK;

	private TableColumn relation, hasInverse, domain, range;

	static public PropertyView theInstance = null;
	
	public static final String ID_VIEW = "it.polito.elite.ontosphere3DPlugin.PropertyView";
	
	public void createPartControl(Composite parent) {
		try {
			Bundle bundle = Platform.getBundle("it.polito.elite.ontosphere3dplugin");
			URL url = this.getClass().getClassLoader().getResource("resources/checkbox_ON.gif");
			//Path check_ON = new Path("icons/checkbox_ON.gif");
			//Path check_OFF = new Path("icons/checkbox_OFF.gif"); 

			//URL url = FileLocator.find(bundle, check_ON, null);
			String filePath = FileLocator.toFileURL(url).getPath();

			imageOK = new Image(parent.getDisplay(), filePath);
			//url = FileLocator.find(bundle, check_OFF, null);
			url = this.getClass().getClassLoader().getResource("resources/checkbox_OFF.gif");
			filePath = FileLocator.toFileURL(url).getPath();
			imageNOK = new Image(parent.getDisplay(), filePath);
		} catch (IOException e) {
			e.printStackTrace();
		}
		
		int style = SWT.SINGLE | SWT.BORDER | SWT.H_SCROLL | SWT.V_SCROLL
				| SWT.FULL_SELECTION | SWT.HIDE_SELECTION;

		table = new Table(parent, style);
		table.setLinesVisible(true);
		table.setHeaderVisible(true);

		this.relation = new TableColumn(table, SWT.NONE);
		relation.setText("Name");
		relation.setWidth(50);
		relation.setToolTipText(relation.getText());

		this.domain = new TableColumn(table, SWT.NONE);
		domain.setText("Domain");
		domain.setWidth(50);
		domain.setToolTipText(domain.getText());

		this.range = new TableColumn(table, SWT.NONE);
		range.setText("Range");
		range.setWidth(50);
		range.setToolTipText(range.getText());

		TableColumn type = new TableColumn(table, SWT.NONE);
		type.setText("Is ObjectProperty?");
		type.setWidth(50);
		type.setToolTipText(type.getText());

		TableColumn isTransitive = new TableColumn(table, SWT.NONE);
		isTransitive.setText("Is Transitive?");
		isTransitive.setWidth(50);
		isTransitive.setToolTipText(isTransitive.getText());
		isTransitive.setAlignment(SWT.CENTER);

		TableColumn isSymmetric = new TableColumn(table, SWT.NONE);
		isSymmetric.setText("Is Symmetric?");
		isSymmetric.setWidth(50);
		isSymmetric.setToolTipText(isSymmetric.getText());

		TableColumn isFunctional = new TableColumn(table, SWT.NONE);
		isFunctional.setText("Is Functional?");
		isFunctional.setWidth(50);
		isFunctional.setToolTipText(isFunctional.getText());

		TableColumn isInvFunctional = new TableColumn(table, SWT.NONE);
		isInvFunctional.setText("Inverse is Functional?");
		isInvFunctional.setWidth(50);
		isInvFunctional.setToolTipText(isInvFunctional.getText());

		this.hasInverse = new TableColumn(table, SWT.NONE);
		hasInverse.setText("Inverse Relation");
		hasInverse.setWidth(50);
		hasInverse.setToolTipText(hasInverse.getText());

		Pane3DScene panel3D = OntoSelectorView.getViewerPanel();
		if (panel3D != null)
			this.updateView(panel3D.model);

		PropertyView.theInstance = this;
	}

	public void setFocus() {	}

	public void updateView(KnowledgeBaseModel model) {
		GC gc = new GC(table);
		int averageCharPixels = gc.getFontMetrics().getAverageCharWidth();
		int maxRelName = 0, maxInverse = 0, maxDomain = 0, maxRange = 0;

		Hashtable<String, RelationType> relTypes = model.getRelationTypes();
		for (String relationName : relTypes.keySet()) {
			TableItem item = new TableItem(table, SWT.BORDER | SWT.CENTER);

			RelationType rt = relTypes.get(relationName);

			item.setText(0, relationName);
			if (relationName.length() > maxRelName)
				maxRelName = relationName.length();

			String domainStr = rt.getDomain().toString();
			item.setText(1, domainStr);
			if (domainStr.length() > maxDomain)
				maxDomain = domainStr.length();

			String rangeStr = "";
			NodeTypeAbstraction node = rt.getRange();
			if (node != null)
				rangeStr = node.toString();
			item.setText(2, rangeStr);
			if (rangeStr.length() > maxRange)
				maxRange = rangeStr.length();

			String inverseRelation = "";
			if (rt.hasInverse())
				inverseRelation = rt.getInverseProperty();
			item.setText(8, inverseRelation);
			if (inverseRelation.length() > maxInverse)
				maxInverse = inverseRelation.length();
			
//			 with this approach is possible to use selectable checkbox. To be used if editing capabilities have to be added...
			// TableEditor editor = new TableEditor (table);  
			// editor.grabHorizontal = editor.grabVertical = true;
			// Button isObjectPropertyButton = new Button(table, SWT.CHECK);
			// isObjectPropertyButton.setSelection(rt.isObjectProperty());
			// isObjectPropertyButton.setBackground(
			// table.getParent().getDisplay().getSystemColor(SWT.COLOR_WHITE) ) ;
			// editor.setEditor(isObjectPropertyButton, item, 3);
			
			if(rt.isObjectProperty())
				item.setImage(3,this.imageOK);
			else
				item.setImage(3,this.imageNOK);
			
			if(rt.isTransitive())
				item.setImage(4,this.imageOK);
			else
				item.setImage(4,this.imageNOK);
			
			if(rt.isSymmetric())
				item.setImage(5,this.imageOK);
			else
				item.setImage(5,this.imageNOK);
			
			if(rt.isFunctional())
				item.setImage(6,this.imageOK);
			else
				item.setImage(6,this.imageNOK);
			
			if(rt.isInverseFunctional())
				item.setImage(7,this.imageOK);
			else
				item.setImage(7,this.imageNOK);

		}
		this.relation.pack();
		this.relation.setWidth((int) (averageCharPixels * maxRelName * 1.5));
		this.hasInverse.pack();
		this.hasInverse.setWidth((int) (averageCharPixels * maxInverse * 1.5));
		this.domain.pack();
		this.domain.setWidth((int) (averageCharPixels * maxDomain));
		this.range.pack();
		this.range.setWidth((int) (averageCharPixels * maxRange));
	}

}
