package it.polito.elite.visualization.ontoviewer.behaviors;

import it.polito.elite.visualization.ontoviewer.models.RelationData;
import it.polito.elite.visualization.ontoviewer.scenes3D.Scene3D;

import java.awt.AWTEvent;
import java.awt.MenuItem;
import java.awt.PopupMenu;
import java.awt.event.InputEvent;
import java.awt.event.MouseEvent;
import java.util.Enumeration;
import java.util.HashSet;

import javax.media.j3d.Behavior;
import javax.media.j3d.BoundingSphere;
import javax.media.j3d.BranchGroup;
import javax.media.j3d.Canvas3D;
import javax.media.j3d.WakeupCriterion;
import javax.media.j3d.WakeupOnAWTEvent;
import javax.media.j3d.WakeupOr;

import com.sun.j3d.utils.geometry.Cone;
import com.sun.j3d.utils.geometry.Primitive;
import com.sun.j3d.utils.picking.PickCanvas;
import com.sun.j3d.utils.picking.PickResult;
import com.sun.j3d.utils.picking.PickTool;

/** Listener click mouse */
public class MyPickBehaviour extends Behavior {

	private Scene3D scene = null;

	protected PickCanvas pickCanvas;

	protected WakeupCriterion[] conditions;

	protected WakeupOr wakeupCondition;

	protected MouseEvent mevent;


	public MyPickBehaviour(Canvas3D canvas, Scene3D scene) {
		super();
		BranchGroup root = scene.getSceneBranchGroup();
		pickCanvas = new PickCanvas(canvas, root);
		this.scene = scene;
		setSchedulingBounds(new BoundingSphere());
		pickCanvas.setMode(PickTool.BOUNDS);
	}

	public void initialize() {
		conditions = new WakeupCriterion[1];
		conditions[0] = new WakeupOnAWTEvent(MouseEvent.MOUSE_CLICKED);
		wakeupCondition = new WakeupOr(conditions);
		wakeupOn(wakeupCondition);
	}

	public void processStimulus(Enumeration criteria) {
		WakeupCriterion wakeup;
		AWTEvent[] evt = null;
		
		while (criteria.hasMoreElements()) {
			wakeup = (WakeupCriterion) criteria.nextElement();
			if (wakeup instanceof WakeupOnAWTEvent) {
				evt = ((WakeupOnAWTEvent) wakeup).getAWTEvent();
			}
			
			for (int i = 0; i < evt.length; i++) {
				if (evt[i] instanceof MouseEvent) {
					mevent = (MouseEvent) evt[i];
					processMouseEvent((MouseEvent) evt[i]);
				}

			}
		}
		wakeupOn(wakeupCondition);
	}

	private void processMouseEvent(MouseEvent evt) {
		if (evt.getID() == MouseEvent.MOUSE_PRESSED
				| evt.getID() == MouseEvent.MOUSE_CLICKED)
			this.updateScene(evt.getX(), evt.getY());
	}
	
	/**
	 * Where the processing is carried out. This checks if an object has been
	 * picked and if it has prints out the user data for that shape.
	 * 
	 * @xpos int that stores the x-position of the mouse
	 * @ypos int that stores the y-position of the mouse
	 */
	public void updateScene(int xpos, int ypos) {
		String uri;
		PickResult pickResult = null;
		Primitive pickedShape = null;

		// Checking if the click has captured any 3D object
		this.pickCanvas.setShapeLocation(xpos, ypos);
		pickResult = pickCanvas.pickClosest();
		if (pickResult != null)
			pickedShape = (Primitive) pickResult.getNode(PickResult.PRIMITIVE);

		// No pickable element found, RClick on the scene centers it
		if (pickedShape == null) {
			if (mevent.getModifiers() == InputEvent.BUTTON3_MASK)
				this.scene.backToInitialSceneView();
		} else {
			
			if (pickedShape instanceof Cone) {//Relations arrow heads
				HashSet<RelationData> set= (HashSet<RelationData>) pickedShape.getUserData();
				this.handlePickedRelationCone(set, xpos, ypos);
			} else{//Concepts, Instances, RelationRanges
				Object o=pickedShape.getUserData();
				uri = o.toString();
				this.handlePickedObject(uri);
			}

		}
	}

	private void handlePickedObject(String uri) {
		int mod = mevent.getModifiers();
		if (mod == InputEvent.BUTTON1_MASK) {// mouse left button
			this.scene.selectElement(uri);
		} else if (mod == InputEvent.BUTTON2_MASK) {// mouse central button
			this.scene.expand_OR_collapseElement(uri);
		} else if (mod == InputEvent.BUTTON3_MASK) {// mouse right button
			this.showContextMenu(uri, mevent.getX(), mevent.getY());
		}
	}

	private void handlePickedRelationCone(HashSet<RelationData> set, int xpos, int ypos) {
			
		int mod = mevent.getModifiers();
		//Shows the relation list
		if (mod == InputEvent.BUTTON1_MASK  || mod == InputEvent.BUTTON2_MASK) {// mouse left button
			PopupMenu menu = new PopupMenu();
			MenuItem title=new MenuItem("RELATIONS LIST:");
			title.setEnabled(false);
			menu.add(title);
			menu.addSeparator();
			for (RelationData reldata : set) {
				MenuItem item = new MenuItem(reldata.toString());
				menu.add(item);
			}
			this.scene.addPopUpMenu(menu);
			menu.show(mevent.getComponent().getParent(), xpos, ypos);
			
		} 
		else if (mod == InputEvent.BUTTON3_MASK) {// mouse right button
			final PopupMenu menu = new PopupMenu();
			MenuItem title=new MenuItem(" OPTIONS:");
			title.setEnabled(false);
			menu.add(title);
			menu.addSeparator();
			
			boolean show = this.scene.showRelationContextMenu(menu, set);
			if(show){
				this.scene.addPopUpMenu(menu);
				menu.show(mevent.getComponent(), xpos, ypos);
			}
		}
	}
	


	private void showContextMenu(final String uri, int xpos, int ypos) {
		final PopupMenu menu = new PopupMenu();
		
		MenuItem title=new MenuItem(uri+" - OPTIONS:");
		title.setEnabled(false);
		menu.add(title);
		menu.addSeparator();
		
		boolean show=this.scene.showContextMenu(menu, uri);
		
		if(show){
			this.scene.addPopUpMenu(menu);
			menu.show(mevent.getComponent(), xpos, ypos);
		}
	}

	public MouseEvent getMevent() {
		return mevent;
	}
	
}
