package it.polito.elite.visualization.ontoviewer.models;

import it.polito.elite.ontology.NodeTypeAbstraction;
import it.polito.elite.visualization.ontoviewer.enums.LinkDirectionEnum;

import java.util.HashSet;

public class RelationData {
	
	private String relationName;
	private LinkDirectionEnum dir;
	private RelationType relationType;
	
	private int fixedCardinality, minCardinality, maxCardinality;
	private boolean isUniversal, hasValue, isSimple, isCardinalityConstrained, isInherited;
	
	NodeTypeAbstraction relatedNode;
	
	static public HashSet<String> extractRelatedNodes(HashSet<RelationData> relData){
		HashSet<String> result=new HashSet<String>(relData.size());
		for(RelationData rd: relData){
			NodeTypeAbstraction node=rd.getRelatedNode();
			if(node != null)
				result.add(node.toString());
		}
		return result;
	}
	
	public RelationData(String relationName, LinkDirectionEnum dir, RelationType relType) {
		this.relationName = new String(relationName);
		this.dir = dir;
		this.relationType = relType;
		if(this.relationType.isFunctional()){
			this.setMaxCardinality(1);
		}
		isSimple=true;
		isInherited=hasValue=isUniversal=isCardinalityConstrained=false;
		fixedCardinality = minCardinality = maxCardinality = -1;
		this.relatedNode = null;
	}
	
	/**
	 * @return Returns the dir.
	 */
	public LinkDirectionEnum getDir() {
		return dir;
	}

	/**
	 * @return Returns the relationName.
	 */
	public String getRelationName() {
		return relationName;
	}

	public int getFixedCardinality() {
		return fixedCardinality;
	}

	public void setFixedCardinality(int fixedCardinality) {
		if(fixedCardinality>0){
			this.isSimple=false;
			this.isCardinalityConstrained=true;
			this.fixedCardinality = fixedCardinality;
		}
	}

	public int getMaxCardinality() {
		return maxCardinality;
	}

	public void setMaxCardinality(int maxCardinality) {
		if(maxCardinality >0){
			this.isSimple=false;
			this.isCardinalityConstrained=true;
			this.maxCardinality = maxCardinality;
		}
	}

	public int getMinCardinality() {
		return minCardinality;
	}

	public void setMinCardinality(int minCardinality) {
		if(minCardinality>0){
			this.isSimple=false;
			this.isCardinalityConstrained=true;
			this.minCardinality = minCardinality;
		}
	}

	public boolean isUniversal() {
		return isUniversal;
	}

	public void setUniversal(boolean isUniversal) {
		this.isSimple=!isUniversal;
		this.isUniversal = isUniversal;
	}

	public boolean hasSameRelatedNode(RelationData r){
		return this.relatedNode.toString().equals(r.relatedNode.toString());
	}
		
	public boolean equals(RelationData r){
		if(this.relationName.equals(r.relationName) &&
			this.isUniversal == r.isUniversal &&
			this.fixedCardinality == r.fixedCardinality &&
			this.maxCardinality == r.maxCardinality &&
			this.minCardinality == r.minCardinality &&
			this.hasValue == r.hasValue	&&
			this.isSimple == r.isSimple){
			
			return true;
		}
		else 
			return false;
	}

	public NodeTypeAbstraction getRelatedNode() {
		return relatedNode;
	}

	public void setRelatedNode(NodeTypeAbstraction relatedNode) {
		this.relatedNode = relatedNode;
	}

	public boolean isHasValue() {
		return hasValue;
	}

	public void setHasValue(boolean hasValue) {
		this.isSimple=!hasValue;
		this.hasValue = hasValue;
	}

	public boolean isSimple() {
		return isSimple;
	}

	public boolean isCardinalityConstrained() {
		return isCardinalityConstrained;
	}

	public boolean isInherited() {
		return isInherited;
	}

	public void setInherited(boolean isInherited) {
		this.isInherited = isInherited;
	}
	
	public String toString(){
		StringBuffer myBuffer=new StringBuffer(this.getRelationName());
		if(isCardinalityConstrained){
			myBuffer.append(" CARD ");
			if(this.fixedCardinality>0)
				myBuffer.append(" = "+this.fixedCardinality);
			else{
				if(this.minCardinality>0)
					myBuffer.append(" > "+(this.minCardinality-1));
				if(this.maxCardinality>0)
					myBuffer.append(" < "+(this.maxCardinality+1));
			}
		}
		else if(isUniversal){
			myBuffer.append(" ALL FROM range");
		}
		
		if(!this.relationType.isSimple()){
			myBuffer.append(" ["+this.relationType.toString()+"]");
		}
		return myBuffer.toString();
	}

	public RelationType getRelationType() {
		return relationType;
	}
}
