package it.polito.elite.visualization.ontoviewer.scenes3D;

import it.polito.elite.visualization.ontoviewer.models.ConceptData;
import it.polito.elite.visualization.ontoviewer.models.InstanceData;
import it.polito.elite.visualization.ontoviewer.models.RelationData;
import it.polito.elite.visualization.ontoviewer.tools.ConceptSphere;
import it.polito.elite.visualization.ontoviewer.tools.InstanceCube;
import it.polito.elite.visualization.ontoviewer.userInterfaces.Pane3DScene;
import it.polito.elite.visualization.ontoviewer.userInterfaces.VisualizationOptions;

import java.awt.MenuItem;
import java.awt.PopupMenu;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.HashSet;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Set;

import javax.media.j3d.BranchGroup;
import javax.media.j3d.Transform3D;
import javax.media.j3d.TransformGroup;
import javax.vecmath.Color3f;
import javax.vecmath.Point3f;
import javax.vecmath.Vector3f;

public class SceneFocusInstances4 extends Scene3D{
	
	private float radiusBackGroundSphere = 0;
	private Set<String> set;
	private BranchGroup fakeBG;
	private Hashtable<String, BranchGroup> instanceCubesBg;
	private Vector3f fakeCubePosition = null;
	
	/* Constructor */
	public SceneFocusInstances4(Pane3DScene _mf, String uri) {
		super(_mf,uri,null,4);
		
		ConceptData cd = this.refMainPanel.model.getConceptData(this.currentConceptURI);
		set = cd.getInstancesURI();
		this.instanceCubesBg = new Hashtable<String, BranchGroup>();
		
		this.createScene();
		this.addBehaviors();
		this.repaintCanvas();
	}

	public SceneFocusInstances4(Pane3DScene _mf, Set<String> uris, String conceptUri) {
		super(_mf,conceptUri,null,4);
		
		this.instanceCubesBg = new Hashtable<String, BranchGroup>();
		this.set=new HashSet<String>();
		this.set.addAll(uris);
		
		this.createScene();
		this.addBehaviors();
		this.repaintCanvas();			
	}
	
	public void createScene() {
		this.addSceneLabel("Instances of "+this.currentConceptURI);
		this.computeNodesPosition();
		this.drawScene();
	}

	private void computeNodesPosition() {
		int numInstances = set.size();

		if (numInstances > 3) { // dispongo su sfera

			int lvl = (int) Math.sqrt(numInstances - 2);
			if ((lvl % 2) == 0) // pari
				lvl--;

			int slice = 1;
			while (((slice * lvl) + 2 < numInstances)) {
				slice++;
			}

			double teta = Math.PI / (lvl + 1);
			double phi = 2 * Math.PI / slice;

			radiusBackGroundSphere = (float) (VisualizationOptions.s1_Segment2ArcCorrectionFactor * 
					(VisualizationOptions.s1_SimpleSphereRadius + VisualizationOptions.sG_distance_min) * slice / (Math.PI));

			Iterator<String> itInstanceURI = set.iterator();
			for (int ni = 1; ni < lvl + 1; ni++) {
				for (int nj = 0; nj < slice; nj++) {

					if (itInstanceURI.hasNext()) {
						String instanceURI = itInstanceURI.next();
							
						Vector3f pos = new Vector3f(.0f, .0f, .0f);
						pos.z += (float) (radiusBackGroundSphere * Math.sin(teta * ni) * Math
								.cos(phi * nj));
						pos.x += (float) (radiusBackGroundSphere * Math.sin(teta * ni) * Math
								.sin(phi * nj));
						pos.y += (float) (radiusBackGroundSphere * Math.cos(teta * ni));

						this.positions.put(instanceURI,new Vector3f(pos));
					}
				}
			}

			int lackingRoots = numInstances - (lvl * slice);

			switch (lackingRoots) {
			case 2: // aggiunge polo sud - manca break, anche 1 viene eseguito
				if (itInstanceURI.hasNext()) {
					String instanceURI = itInstanceURI.next();

					Vector3f pos = new Vector3f(.0f, .0f, .0f);
					pos.y += (float) (-1) * radiusBackGroundSphere;

					this.positions.put(instanceURI,new Vector3f(pos));
				}
			case 1: // aggiunge polo nord
				if (itInstanceURI.hasNext()) {
					String instanceURI = itInstanceURI.next();

					Vector3f pos = new Vector3f(.0f, .0f, .0f);
					pos.y += (float) radiusBackGroundSphere;

					this.positions.put(instanceURI,new Vector3f(pos));
				}
				break;
			}
		} else if (numInstances == 1) { // un solo root lo disegno nell'origine
			for (String instanceURI : set) {
				this.positions.put(instanceURI,new Vector3f(.0f, .0f, .0f));
			}
		} else { // se 2 o 3 li dispongo sulla circonferenza

			// setto centro circonferenza
			double alfa = (2 * Math.PI) / numInstances;
			radiusBackGroundSphere = (float) (0.1f + (2 * (VisualizationOptions.s1_SimpleSphereRadius + VisualizationOptions.sG_distance_min) * numInstances)
					/ (2 * Math.PI));

			int ni = 0;

			for (String instanceURI : set) {
				Vector3f pos = new Vector3f(.0f, .0f, .0f);
				pos.z += (float) (radiusBackGroundSphere * Math.sin(alfa * ni));
				pos.x += (float) (radiusBackGroundSphere * Math.cos(alfa * ni));
				pos.y = 0;
				ni++;

				this.positions.put(instanceURI,new Vector3f(pos));
			}
		}
	}

	protected void drawScene() {
		this.drawNodes(set);

		if (this.radiusBackGroundSphere > 0) {
			// crea la sfera e la aggiunge al tgx
			ConceptSphere sfera = new ConceptSphere(new Color3f(.352f, .745f, .9f),
					null, false, false);
			sfera.setTransparency(.5f);
			sfera.setRadius(radiusBackGroundSphere);
			tgSCENE.addChild(sfera.createColoredSphere());
		}
	}

	public BranchGroup drawInstance(String uri, InstanceData data) {

		BranchGroup bgx = this.createBranchGroup();
		TransformGroup tgx = this.createTransformGroup();
		Vector3f center = this.positions.get(uri);

		// posiziona (transla il transform group)
		Transform3D transf = new Transform3D();
		transf.setTranslation(center);
		tgx.setTransform(transf);

		// crea il cubo e lo aggiunge al tgx
		Color3f color;
		int n_facts=data.getIncomingLinks().size()+data.getOutgoingLinks().size();
		if (n_facts>8)
			n_facts=8;
		
		if(n_facts==0){
			color = new Color3f(VisualizationOptions.s4_InstanceWithoutFactsColor);
		}
		else
			color = new Color3f(VisualizationOptions.s4_InstanceWithFactsColor);
		
		InstanceCube cubo = new InstanceCube();

		float side;
		if(n_facts>0)
			cubo.setLabelAppearance(VisualizationOptions.s4_LabelInstanceWithChildren,
					VisualizationOptions.s4_LabelInstanceScale);
		else
			cubo.setLabelAppearance(VisualizationOptions.s4_LabelInstanceNoChild,
					VisualizationOptions.s4_LabelInstanceScale);
		
		side = VisualizationOptions.s4_cubeSide+
			(VisualizationOptions.s4_InstanceSideMultiplier*n_facts);
		if(side>VisualizationOptions.s4_InstanceSideLimit)
			side=VisualizationOptions.s4_InstanceSideLimit;

		cubo.setSide(side);


		BranchGroup bgCube = cubo.createColoredCube(color, uri, data.hasPertainingFacts());
		this.instanceCubesBg.put(uri, bgCube);
		tgx.addChild(bgCube);
		bgx.addChild(tgx);
		return bgx;
	}
	
	private void drawNodes(Set<String> set) {
		for (String uriInstance : set) {
			InstanceData idata = this.refMainPanel.model.getInstanceData(uriInstance);
			this.tgSCENE.addChild(drawInstance(uriInstance, idata));
		}
	}

	@Override
	public boolean showContextMenu(final PopupMenu menu, final String uri) {
		MenuItem item1 = new MenuItem("GoTo Concept");
		item1.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				SceneFocusConcept3 myScene=new SceneFocusConcept3(refMainPanel,currentConceptURI);
				refMainPanel.changeScene(myScene);
				removePopUpMenu(menu);
			}
		});
		menu.add(item1);
		return true;
	}

	@Override
	public void selectElement(String uri) {
		//if(this.refMainPanel.model.getInstanceData(uri).hasPertainingFacts()){
			SceneFocusFacts5 myScene=new SceneFocusFacts5(refMainPanel,currentConceptURI,uri);
			this.refMainPanel.changeScene(myScene);
		//}
	}

	@Override
	public void expand_OR_collapseElement(String uri) {	}

	@Override
	public boolean showRelationContextMenu(PopupMenu menu, HashSet<RelationData> relSet) {	return false; }

	@Override
	public void fadeInAnimation() {
			switch(this.refMainPanel.getOldSceneID()){
			default:
				if(refMainPanel.isPrimary)
					this.refMainPanel.primaryBG.removeAllChildren();
				else
					this.refMainPanel.secondaryBG.removeAllChildren();
			}
	}

	@Override
	public void fadeOutAnimation(String uri, Scene3D nextScene) {
//		Disables interaction with the scene
		this.removeBehaviours();
		// create the animations
		BranchGroup bgAnimation = createBranchGroup();
		InstanceData data=this.refMainPanel.model.getInstanceData(uri);
		TransformGroup fakeTG = this.createFakeInstanceCube(data);
		long time = System.currentTimeMillis();
		Vector3f center = this.positions.get(uri);
		this.instanceCubesBg.get(uri).detach();
		
		Vector3f finalPosition = new Vector3f(.0f,.0f,.0f);
		
		//fadeOutMainSphere(bgAnimation, time, center);
		//fadeOutFakeSphere(bgAnimation, time, finalPosition, fakeTG,	nextScene);

		fakeBG.addChild(bgAnimation);
		
	}
	
	/**
	 * The fake sphere is embedded in 2 transformGroups because of the positionInterpolator.
	 * fakeTG2 deals with the absolute positioning of the sphere, while fakeTG is used for the 
	 * relative movement from initial position to final one...
	 * @param original
	 * @return
	 */
	private TransformGroup createFakeInstanceCube(InstanceData original) {
		// Create a fake concept sphere used for the animation after the scene
		// is faded
		Color3f fakeCol = VisualizationOptions.s4_InstanceWithFactsColor;
		InstanceCube fake = new InstanceCube();
		fake.setSide(original.getInstanceCubeSide());
		//fake.setTransparency(0.05f);


		fakeBG = this.createBranchGroup();
		TransformGroup fakeTG = this.createTransformGroup();
		TransformGroup fakeTG2 = this.createTransformGroup();
		
		//Computes the position of the fake conceptSphere
		Point3f point=new Point3f(this.positions.get(original.getUri()));
		Transform3D currentTransform = new Transform3D();
		this.tgROT.getTransform(currentTransform);
		currentTransform.transform(point);
		Transform3D translation=new Transform3D();
		fakeCubePosition = new Vector3f(point);
		translation.setTranslation(fakeCubePosition);
		
		fakeTG2.setTransform(translation);
		fakeTG.addChild(fake.createColoredCube(fakeCol, original.getUri(), true));
		fakeTG2.addChild(fakeTG);
		fakeBG.addChild(fakeTG2);
		if(this.refMainPanel.isPrimary)
			this.refMainPanel.secondaryBG.addChild(fakeBG);
		else
			this.refMainPanel.primaryBG.addChild(fakeBG);
		return fakeTG;
	}

}
