package it.polito.elite.visualization.ontoviewer.scenes3D;

import it.polito.elite.listeners.PostAnimation_GroupDetaching;
import it.polito.elite.listeners.PostAnimation_SceneChanger;
import it.polito.elite.ontology.NodeTypeAbstraction;
import it.polito.elite.visualization.ontoviewer.behaviors.MyMouseCursorSelector;
import it.polito.elite.visualization.ontoviewer.behaviors.MyMouseTranslate;
import it.polito.elite.visualization.ontoviewer.behaviors.MyPickBehaviour;
import it.polito.elite.visualization.ontoviewer.behaviors.PositionInterpolatorWithListener;
import it.polito.elite.visualization.ontoviewer.behaviors.TransparencyInterpolatorWithListener;
import it.polito.elite.visualization.ontoviewer.enums.LinkDirectionEnum;
import it.polito.elite.visualization.ontoviewer.enums.NodeTypesEnum;
import it.polito.elite.visualization.ontoviewer.models.ConceptData;
import it.polito.elite.visualization.ontoviewer.models.RelationData;
import it.polito.elite.visualization.ontoviewer.tools.ConceptSphere;
import it.polito.elite.visualization.ontoviewer.tools.Relation3DObject;
import it.polito.elite.visualization.ontoviewer.userInterfaces.Pane3DScene;
import it.polito.elite.visualization.ontoviewer.userInterfaces.VisualizationOptions;

import java.awt.Color;
import java.awt.Font;
import java.awt.MenuItem;
import java.awt.PopupMenu;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.HashSet;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Set;

import javax.media.j3d.Alpha;
import javax.media.j3d.Appearance;
import javax.media.j3d.BoundingSphere;
import javax.media.j3d.BranchGroup;
import javax.media.j3d.ScaleInterpolator;
import javax.media.j3d.Shape3D;
import javax.media.j3d.Transform3D;
import javax.media.j3d.TransformGroup;
import javax.media.j3d.TransparencyAttributes;
import javax.media.j3d.TransparencyInterpolator;
import javax.swing.JColorChooser;
import javax.vecmath.AxisAngle4f;
import javax.vecmath.Color3f;
import javax.vecmath.Point3d;
import javax.vecmath.Point3f;
import javax.vecmath.Vector3f;

import com.sun.j3d.utils.behaviors.mouse.MouseRotate;
import com.sun.j3d.utils.behaviors.mouse.MouseWheelZoom;

public class SceneFocusRoots1 extends Scene3D {

	float radius = 0, originalSphereRadius=0;

	private ConceptSphere bgSphere;

	private BranchGroup fakeBG;

	private HashSet<String> workingSet;

	private Hashtable<String, BranchGroup> conceptSpheresBg;
	
	private Vector3f fakeSpherePosition = null;
	
	public SceneFocusRoots1(Pane3DScene _mf, Set<String> uris) {
		super(_mf, null, null, 1);

		this.workingSet = new HashSet<String>();
		this.conceptSpheresBg = new Hashtable<String, BranchGroup>();
		if (uris != null && uris.size() > 0) {
			this.workingSet.addAll(uris);
			this.refMainPanel.conceptsOfCurrentLogicView = this.workingSet;
			if(this.refMainPanel.currentLogicViewName != null){
				this.addSceneLabel(this.refMainPanel.currentLogicViewName+ " view");
			}
		} else{
			this.workingSet.addAll(this.refMainPanel.model.getRoots());
			this.refMainPanel.conceptsOfCurrentLogicView = this.workingSet;
		}
		this.createScene();
		this.addBehaviors();
	}

	public void createScene() {
		this.computeNodesPosition(workingSet.size());
		this.drawScene();
	}

	private void computeNodesPosition(int numNodes) {

		if (numNodes > 3) { // dispongo su sfera

			int lvl = (int) Math.sqrt(numNodes - 2);
			if ((lvl % 2) == 0) // pari
				lvl--;

			int slice = 1;
			while (((slice * lvl) + 2 < numNodes)) {
				slice++;
			}

			double teta = Math.PI / (lvl + 1);
			double phi = 2 * Math.PI / slice;
			// segment->arc approx = k * numSphere*(SphereRad + Dist)/Pi
			radius = (float) (VisualizationOptions.s1_Segment2ArcCorrectionFactor
					* (VisualizationOptions.s1_SimpleSphereRadius + VisualizationOptions.sG_distance_min)
					* slice / (Math.PI));

			Iterator<String> it = workingSet.iterator();
			for (int ni = 1; ni < lvl + 1; ni++) {
				for (int nj = 0; nj < slice; nj++) {

					if (it.hasNext()) {
						String conceptUri = it.next();

						Vector3f pos = new Vector3f(.0f, .0f, .0f);
						pos.z += (float) (radius * Math.sin(teta * ni) * Math
								.cos(phi * nj));
						pos.x += (float) (radius * Math.sin(teta * ni) * Math
								.sin(phi * nj));
						pos.y += (float) (radius * Math.cos(teta * ni));

						this.positions.put(conceptUri, new Vector3f(pos));
					}
				}
			}

			int lackingRoots = numNodes - (lvl * slice);

			switch (lackingRoots) {
			case 2: // aggiunge polo sud - manca break, anche 1 viene eseguito
				if (it.hasNext()) {
					String conceptUri = it.next();
					this.positions.put(conceptUri, new Vector3f(.0f, (-1)
							* radius, .0f));
				}
			case 1: // aggiunge polo nord
				if (it.hasNext()) {
					String conceptUri = it.next();
					this.positions.put(conceptUri, new Vector3f(.0f, radius,
							.0f));
				}
				break;
			}
		} else if (numNodes == 1) { // un solo root lo disegno nell'origine
			for (String conceptUri : workingSet) {
				this.positions.put(conceptUri, new Vector3f(.0f, .0f, .0f));
			}
		} else { // se 2 o 3 li dispongo sulla circonferenza

			// setto centro circonferenza
			double alfa = (2 * Math.PI) / numNodes;
			radius = (float) (0.1f + (2 * (VisualizationOptions.s1_SimpleSphereRadius + VisualizationOptions.sG_distance_min) * numNodes)
					/ (2 * Math.PI));

			int ni = 0;

			for (String conceptUri : workingSet) {
				Vector3f pos = new Vector3f(.0f, .0f, .0f);
				pos.z += (float) (radius * Math.sin(alfa * ni));
				pos.x += (float) (radius * Math.cos(alfa * ni));
				pos.y = 0;
				ni++;

				this.positions.put(conceptUri, pos);
			}
		}

	}

	private void drawScene() {
		this.drawNodes();

		if (this.radius > 0) {
			// crea la sfera e la aggiunge al tgx
			this.bgSphere = new ConceptSphere(
					VisualizationOptions.s1_MainSphereColor, null, false, false);
			bgSphere
					.setTransparency(VisualizationOptions.s1_MainSphereTransparency);
			bgSphere.setRadius(radius);
			tgSCENE.addChild(bgSphere.createColoredSphere());
		}
	}

	private BranchGroup drawConcept(String uri, ConceptData data) {

		BranchGroup bgx = this.createBranchGroup();
		TransformGroup tgx = this.createTransformGroup();
		Vector3f center = this.positions.get(uri);
		boolean hasSons = false;
		if (data.getNumberOfChildren() > 0) // se ha figli
			hasSons = true;

		// creating the sphere and adding it to the tgx
		Color3f color = null;
		if (hasSons) // if has children
			color = VisualizationOptions.s1_ParentSphereColor;
		else
			color = VisualizationOptions.s1_SimpleSphereColor;

		ConceptSphere sfera = new ConceptSphere(color, uri, true, data
				.hasInstances());

		float radius;
		if (hasSons) { // if has children
			sfera.setLabelColor(VisualizationOptions.s1_LabelParentSphereColor);
			sfera.setLabelScale(VisualizationOptions.s1_LabelParentSphereScale);
			radius = VisualizationOptions.s1_SimpleSphereRadius	+ data.getSubTreeDepth()
					* VisualizationOptions.s1_ParentSphereRadiusMultiplier;
			if ((radius > VisualizationOptions.s1_ParentSphereRadiusLimit)
					&& (VisualizationOptions.s1_ParentSphereRadiusLimit > 0))
				radius = VisualizationOptions.s1_ParentSphereRadiusLimit;
		} else {
			sfera.setLabelColor(VisualizationOptions.s1_LabelSimpleSphereColor);
			sfera.setLabelScale(VisualizationOptions.s1_LabelSimpleSphereScale);
			radius = VisualizationOptions.s1_SimpleSphereRadius;
		}
		sfera.setRadius(radius);
		data.setConceptSphereRadius(radius);


//groups relations towards element on screen in an Hashtable with key: endNode URI 
//and as Obj: the relations directed towards that node and starting from the concept drawn
		Hashtable<String,HashSet<RelationData>> outRelationOnScreen=new Hashtable<String, HashSet<RelationData>>();
		for (String relUri : data.getOutgoingLinks().keySet()) {
			HashSet<RelationData> rangeSet = data.getOutgoingLinks().get(relUri);
			
			for(RelationData myRelationData: rangeSet){
				NodeTypeAbstraction myRange=myRelationData.getRelatedNode();
				//we want to draw only relations towards named classes
				if(myRange!=null && myRange.getType()==NodeTypesEnum.NamedClass){
					String endUri=myRange.toString();
					//and only on screen
					if (workingSet.contains(endUri)) {
						if(outRelationOnScreen.containsKey(endUri)){
							HashSet<RelationData> relations=outRelationOnScreen.get(endUri);
							relations.add(myRelationData);
						}
						else{
							HashSet<RelationData> relations=new HashSet<RelationData>(2,2);
							relations.add(myRelationData);
							outRelationOnScreen.put(endUri,relations);
						}
					}
				}	
			}
		}
		// draws the retrieved outgoing relations towards element in the Scene
		for(String endUri: outRelationOnScreen.keySet()){
			HashSet<RelationData> relationsSet=outRelationOnScreen.get(endUri);
			Vector3f posEndUri = this.positions.get(endUri);
			Vector3f relationEnd = new Vector3f(posEndUri.x - center.x,
					posEndUri.y - center.y, posEndUri.z - center.z);
			Relation3DObject relation = new Relation3DObject();
			tgx.addChild(relation.createStraightRelation(relationsSet,
					relationEnd, data.getConceptSphereRadius(),	LinkDirectionEnum.out));
		}
		
		if (data.getSelfLoopLinks().size() > 0) {
			Relation3DObject relation = new Relation3DObject();
			// Define end points as new vectors
			Vector3f fromDir = new Vector3f(1f, 0f, 0f);
			Vector3f toDir = new Vector3f(center);
			toDir.normalize();
			AxisAngle4f rotAxis;
			if ((toDir.equals(fromDir) || toDir	.equals(new Vector3f(-1f, 0f, 0f)))) {
				rotAxis = new AxisAngle4f();
			} else {
				// Compute the value of the angle between them (no spatial
				// orientation info though)
				float rotInRadians = fromDir.angle(toDir);
				// Compute an orthagonal vector to later define the rotation
				// axis
				Vector3f orthagonalAxis = new Vector3f();
				orthagonalAxis.cross(fromDir, toDir);
				orthagonalAxis.normalize();
				// Create new rotation matrix
				rotAxis = new AxisAngle4f(orthagonalAxis, rotInRadians);
			}

			tgx.addChild(relation.createSelfLoopRelation(data.getSelfLoopLinks(), data
							.getConceptSphereRadius(), rotAxis));
		}

		// posiziona (transla il transform group)
		Transform3D transf = new Transform3D();
		transf.setTranslation(center);
		tgx.setTransform(transf);
		BranchGroup bgSphere = sfera.createColoredSphere();
		this.conceptSpheresBg.put(uri, bgSphere);
		tgx.addChild(bgSphere);
		bgx.addChild(tgx);
		return bgx;
	}

	private void drawNodes() {
		for (String uri : workingSet) {
			ConceptData cd = this.refMainPanel.model.getConceptData(uri);
			this.tgSCENE.addChild(drawConcept(uri, cd));
		}
	}
	
	///////////////////////////////SCENE INTERACTION RELATED METHODS
	
	@Override
	public void selectElement(String newUri) {
		ConceptData original=this.refMainPanel.model.getConceptData(newUri);
		this.originalSphereRadius = original.getConceptSphereRadius();
		Scene3D myScene = new SceneFocusConcept3(this.refMainPanel, newUri);

		if (VisualizationOptions.sG_useAnimations) {
			this.fadeOutAnimation(newUri, myScene);
		} else {
			this.refMainPanel.changeScene(myScene);
		}
	}

	@Override
	public boolean showContextMenu(final PopupMenu menu, final String uri) {
		final ConceptData cd = this.refMainPanel.model.getConceptData(uri);
		
		if (this.refMainPanel.model.getConceptData(uri).getNumberOfChildren() > 0){
			MenuItem item = new MenuItem("Explore Subtree");
			item.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
				ConceptData original=refMainPanel.model.getConceptData(uri);
				originalSphereRadius = original.getConceptSphereRadius();
				Scene3D myScene = new SceneFocusTree2(refMainPanel, uri);
				
				if (VisualizationOptions.sG_useAnimations) {
					fadeOutAnimation(uri, myScene);
				} else {
					refMainPanel.changeScene(myScene);
				}
				removePopUpMenu(menu);
			}});
			menu.add(item);
		}

		MenuItem item = new MenuItem("Explore Concept");
		item.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
			ConceptData original=refMainPanel.model.getConceptData(uri);
			originalSphereRadius = original.getConceptSphereRadius();
			Scene3D myScene = new SceneFocusConcept3(refMainPanel, uri);

			if (VisualizationOptions.sG_useAnimations) {
				fadeOutAnimation(uri, myScene);
			} else {
				refMainPanel.changeScene(myScene);
			}
			removePopUpMenu(menu);
		}});
		menu.add(item);
		
		if (cd.hasInstances()) {
			MenuItem item0 = new MenuItem("Show Direct Instances");
			item0.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					SceneFocusInstances4 myScene = new SceneFocusInstances4(refMainPanel, uri);
					if (VisualizationOptions.sG_useAnimations) {
						fadeOutAnimation(uri, myScene);
					} else {
						refMainPanel.changeScene(myScene);
					}
					removePopUpMenu(menu);
				}
			});
			menu.add(item0);
		}
		
		if(cd.hasNonDirectInstances()){
			MenuItem item1 = new MenuItem("Show All Instances");
			item1.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					Set<String> set = refMainPanel.model.retrieveAllInstancesOf(uri);
					SceneFocusInstances4 myScene = new SceneFocusInstances4(refMainPanel, set, uri);
					
					if (VisualizationOptions.sG_useAnimations) {
						fadeOutAnimation(uri, myScene);
					} else {
						refMainPanel.changeScene(myScene);
					}
					removePopUpMenu(menu);
				}
			});
			menu.add(item1);
		}
		
		if(cd.getChildren().size()>0){
			MenuItem item2 = new MenuItem("Expand Concept into Children");
			item2.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					expandOneConcepts(uri);
					removePopUpMenu(menu);
				}
			});
			menu.add(item2);
		}
		
		MenuItem item3 = new MenuItem("Remove From View");
		item3.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				removeOneConcepts(uri);
				removePopUpMenu(menu);
			}
		});
		menu.add(item3);
		
		
		String parent=cd.getParent();
		if(parent != null){
			boolean hasSiblingOnScene=false;
			ConceptData cdParent=this.refMainPanel.model.getConceptData(parent);
			for(String uriSibling: cdParent.getChildren()){
				if(!uri.equals(uriSibling) && this.workingSet.contains(uriSibling))
					hasSiblingOnScene=true;
			}
			if(hasSiblingOnScene){
				MenuItem item4 = new MenuItem("Collapse with Siblings into Parent");
				item4.addActionListener(new ActionListener() {
					public void actionPerformed(ActionEvent e) {
						collapseSiblings(uri);
						removePopUpMenu(menu);
					}
				});
				menu.add(item4);
			}
		}
		
		MenuItem item5 = new MenuItem("Expand All Layout Concepts into Children");
		item5.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				expandAllConcepts();
				removePopUpMenu(menu);
			}
		});
		menu.add(item5);

		return true;
	}

	@Override
	public void expand_OR_collapseElement(String uri) {
		if (this.refMainPanel.model.getConceptData(uri).getNumberOfChildren() > 0){
			ConceptData original=this.refMainPanel.model.getConceptData(uri);
			this.originalSphereRadius = original.getConceptSphereRadius();
			Scene3D myScene = new SceneFocusTree2(this.refMainPanel, uri);
			
			if (VisualizationOptions.sG_useAnimations) {
				this.fadeOutAnimation(uri, myScene);
			} else {
				this.refMainPanel.changeScene(myScene);
			}
		}
	}

	@Override
	public boolean showRelationContextMenu(final PopupMenu menu,	HashSet<RelationData> relSet) { 
		PopupMenu setColor=new PopupMenu("Set Color To");
		menu.add(setColor);
		boolean found=false;
		for (RelationData reldata : relSet) {
			final String relname=reldata.getRelationName();
			final LinkDirectionEnum dir=reldata.getDir();
			MenuItem item = new MenuItem(relname);
			item.setFont(new Font("Arial",Font.BOLD|Font.ITALIC,12));
			item.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					Color c = JColorChooser.showDialog(null, "Choose a color...", Color.WHITE);
					refMainPanel.model.setColorToRelation(relname, c);
					removePopUpMenu(menu);
				}
			});
			setColor.add(item);
			if(Relation3DObject.getRelationColor(relname)!=null)
				found=true;
		}
		
		if(found){
			PopupMenu unsetColor=new PopupMenu("Remove Custom Color");
			menu.add(unsetColor);
			for (RelationData reldata : relSet) {
				final String relname=reldata.getRelationName();
				if(Relation3DObject.getRelationColor(relname)!=null){
					final LinkDirectionEnum dir=reldata.getDir();
					MenuItem item = new MenuItem(relname);
					item.setFont(new Font("Arial",Font.BOLD|Font.ITALIC,12));
					item.addActionListener(new ActionListener() {
						public void actionPerformed(ActionEvent e) {
							refMainPanel.model.removeColorToRelation(relname);
							removePopUpMenu(menu);
						}
					});
					unsetColor.add(item);
				}
			}
		}
		
		return true; 
	}

	private void removeOneConcepts(String uri) {
		workingSet.remove(uri);
		SceneFocusRoots1 myScene = new SceneFocusRoots1(this.refMainPanel,workingSet);
		this.refMainPanel.changeScene(myScene);
	}

	private void collapseSiblings(String uri) {
		ConceptData cd = this.refMainPanel.model.getConceptData(uri);
		String uriParent = cd.getParent();
		if (uriParent != null) {
			cd = this.refMainPanel.model.getConceptData(uriParent);
			for (String child : cd.getChildren()) {
				workingSet.remove(child);
			}
			workingSet.add(uriParent);
		}
		SceneFocusRoots1 myScene = new SceneFocusRoots1(this.refMainPanel, 	workingSet);
		this.refMainPanel.changeScene(myScene);
	}

	private void expandOneConcepts(String uri) {
		ConceptData cd = this.refMainPanel.model.getConceptData(uri);
		Set<String> children = cd.getChildren();
		if (children.size() > 0) {
			workingSet.addAll(children);
			workingSet.remove(uri);
		}
		SceneFocusRoots1 myScene = new SceneFocusRoots1(this.refMainPanel,
				workingSet);
		this.refMainPanel.changeScene(myScene);
	}

	private void expandAllConcepts() {
		Set<String> resSet = new HashSet<String>();
		for (String uri : workingSet) {
			ConceptData cd = this.refMainPanel.model.getConceptData(uri);
			resSet.addAll(cd.getChildren());
		}
		workingSet.clear();
		workingSet.addAll(resSet);
		SceneFocusRoots1 myScene = new SceneFocusRoots1(this.refMainPanel,
				workingSet);
		this.refMainPanel.changeScene(myScene);
	}
	
	///////////////////////////////ANIMATIONS RELATED METHODS

	@Override
	public void fadeInAnimation() {
		//Disable fading in on Startup and on expanding elm of scene 1
		if(this.refMainPanel.getOldSceneID()<2){
			BranchGroup bg=new BranchGroup();
			bg.addChild(tgROT);
			this.bgSCENE.addChild(bg);
			return;
		}
		
		// create the animations
		BranchGroup bgAnimation = createBranchGroup();
		long time = System.currentTimeMillis();

		// timer for scale interpolator (Increasing)
		Alpha alpha = new Alpha(1, VisualizationOptions.sG_AnimationDuration/2);
		alpha.setStartTime(time);
		alpha.setIncreasingAlphaRampDuration(alpha.getIncreasingAlphaDuration() / 3);

		ScaleInterpolator si = new ScaleInterpolator(alpha, tgSCENE);
		si.setSchedulingBounds(new BoundingSphere());
		bgAnimation.addChild(si);
		
		//Adding element to the scene
		tgSCENE.addChild(bgAnimation);
		BranchGroup bg=new BranchGroup();
		bg.addChild(tgROT);
		this.bgSCENE.addChild(bg);
		}
	
	public void fadeOutAnimation(String uri, Scene3D nextScene) {
		//Disables interaction with the scene
		this.removeBehaviours();
		// create the animations
		BranchGroup bgAnimation = createBranchGroup();
		ConceptData data=this.refMainPanel.model.getConceptData(uri);
		long time = System.currentTimeMillis();
		Vector3f center = this.positions.get(uri);
		
		if(nextScene.sceneID == 4 ){
			refMainPanel.changeScene(nextScene);
		}
		else{
			TransformGroup fakeTG = this.createFakeConceptSphere(data);
			this.conceptSpheresBg.get(uri).detach();
			
			Vector3f finalPosition = new Vector3f(.0f,.0f,.0f);
			if(nextScene.getSceneID() == 2){
				finalPosition = this.centralPositionAtGivenHeightScale(
						VisualizationOptions.s2_ScreenPercent_as_RootVerticalPosition);
			}
			
			fadeOutMainSphere(bgAnimation, time, center);
			fadeOutFakeSphere(bgAnimation, time, finalPosition, fakeTG,	nextScene);
		}
		
		fakeBG.addChild(bgAnimation);
	}

	private void fadeOutMainSphere(BranchGroup bgAnimation, long time,
			Vector3f center) {
		// timer for transparency interpolation (Increasing)
		Alpha alpha = new Alpha(1,
				VisualizationOptions.sG_AnimationDuration * 3 / 4);
		alpha.setStartTime(time);
		alpha.setIncreasingAlphaRampDuration(alpha
						.getIncreasingAlphaDuration() / 3);

		Shape3D array[] = Scene3D.getShapes(this.tgSCENE);
		for (Shape3D shape : array) {

			TransparencyAttributes ta = shape.getAppearance()
					.getTransparencyAttributes();
			if (ta == null) {
				Appearance app = shape.getAppearance();
				ta = new TransparencyAttributes(TransparencyAttributes.BLENDED,
						0f);
				ta.setCapability(TransparencyAttributes.ALLOW_VALUE_READ);
				ta.setCapability(TransparencyAttributes.ALLOW_VALUE_WRITE);
				try {
					app.setTransparencyAttributes(ta);
				} catch (Exception e1) {
					System.err
							.println("Fade out sc1: Error in setting transparency attributes"
									+ shape.getClass());
				}

			}
			TransparencyInterpolator ti = new TransparencyInterpolator(alpha,
					ta, ta.getTransparency(), 1f);
			ti.setSchedulingBounds(new BoundingSphere());
			bgAnimation.addChild(ti);
		}
		// timer for scale interpolator (decreasing)
		Alpha beta = new Alpha(1, 0);
		beta.setDecreasingAlphaDuration(VisualizationOptions.sG_AnimationDuration * 3 / 4);
		beta.setDecreasingAlphaRampDuration(beta
						.getDecreasingAlphaDuration() / 3);
		beta.setStartTime(time);
		beta.setMode(Alpha.DECREASING_ENABLE);

		Transform3D transf = computeOrientation(center);
		ScaleInterpolator si = new ScaleInterpolator(beta, tgSCENE);
		try {
			si.setTransformAxis(transf);
		} catch (javax.vecmath.SingularMatrixException e) {
			 si = new ScaleInterpolator(beta, tgSCENE);
		}
		si.setSchedulingBounds(new BoundingSphere());
		bgAnimation.addChild(si);
	}

	private void fadeOutFakeSphere(BranchGroup bgAnimation, long time,
			Vector3f finalPosition, TransformGroup fakeTG,
			Scene3D nextScene) {
		// timer for position interpolator (increasing)
		Alpha gamma = new Alpha(1,	VisualizationOptions.sG_AnimationDuration / 4);
		gamma.setStartTime(time + VisualizationOptions.sG_AnimationDuration * 3	/ 4);
		gamma.setIncreasingAlphaRampDuration(gamma.getIncreasingAlphaDuration() / 3);
		
		finalPosition.sub(this.fakeSpherePosition);
		float translLenght = finalPosition.length();
		Transform3D transf = computeOrientation(finalPosition);

		PostAnimation_SceneChanger change = new PostAnimation_SceneChanger(
				this.refMainPanel, nextScene, false);
		PositionInterpolatorWithListener piFinal = new PositionInterpolatorWithListener(
				gamma, fakeTG, transf, 0f , translLenght, change);
		piFinal.setSchedulingBounds(new BoundingSphere());
		bgAnimation.addChild(piFinal);

		Alpha delta = new Alpha(1,
				VisualizationOptions.sG_AnimationDuration * 1 / 2);
		delta.setStartTime(time + VisualizationOptions.sG_AnimationDuration);
		delta.setIncreasingAlphaRampDuration(delta
						.getIncreasingAlphaDuration() / 3);
		// Fading Away
		Shape3D array[] = Scene3D.getShapes(this.fakeBG);
		for (Shape3D shape : array) {
			TransparencyAttributes ta = shape.getAppearance()
					.getTransparencyAttributes();

			if (ta == null) {
				Appearance app = shape.getAppearance();
				ta = new TransparencyAttributes(TransparencyAttributes.BLENDED,
						0f);
				ta.setCapability(TransparencyAttributes.ALLOW_VALUE_READ);
				ta.setCapability(TransparencyAttributes.ALLOW_VALUE_WRITE);
				try {
					app.setTransparencyAttributes(ta);
				} catch (Exception e1) {
					System.err.println("Fade out sc1 fakeSph: Error in setting transparency attributes");
				}
			}

			PostAnimation_GroupDetaching cleaner = new PostAnimation_GroupDetaching(
					this.fakeBG);
			TransparencyInterpolatorWithListener ti = new TransparencyInterpolatorWithListener(
					delta, ta, ta.getTransparency(), 1f, cleaner);
			ti.setSchedulingBounds(new BoundingSphere());
			bgAnimation.addChild(ti);
		}
	}
	
	/**
	 * The fake sphere is embedded in 2 transformGroups because of the positionInterpolator.
	 * fakeTG2 deals with the absolute positioning of the sphere, while fakeTG is used for the 
	 * relative movement from initial position to final one...
	 * @param original
	 * @return
	 */
	private TransformGroup createFakeConceptSphere(ConceptData original) {
		// Create a fake concept sphere used for the animation after the scene
		// is faded
		Color3f fakeCol;
		boolean hasSons = false;
		if (original.getNumberOfChildren() > 0) { // se ha figli
			hasSons = true;
			fakeCol = VisualizationOptions.s1_ParentSphereColor;
		} else
			fakeCol = VisualizationOptions.s1_SimpleSphereColor;

		ConceptSphere fake = new ConceptSphere(fakeCol, original.getUri(),
				false, original.hasInstances());
		fake.setRadius(this.originalSphereRadius);
		fake.setTransparency(0.05f);

		if (hasSons) { // se ha figli
			fake.setLabelColor(VisualizationOptions.s1_LabelParentSphereColor);
			fake.setLabelScale(VisualizationOptions.s1_LabelParentSphereScale);
		} else {
			fake.setLabelColor(VisualizationOptions.s1_LabelSimpleSphereColor);
			fake.setLabelScale(VisualizationOptions.s1_LabelSimpleSphereScale);
		}
		fakeBG = this.createBranchGroup();
		TransformGroup fakeTG = this.createTransformGroup();
		TransformGroup fakeTG2 = this.createTransformGroup();
		
		//Computes the position of the fake conceptSphere
		Point3f point=new Point3f(this.positions.get(original.getUri()));
		Transform3D currentTransform = new Transform3D();
		this.tgROT.getTransform(currentTransform);
		currentTransform.transform(point);
		Transform3D translation=new Transform3D();
		fakeSpherePosition = new Vector3f(point);
		translation.setTranslation(fakeSpherePosition);
		
		fakeTG2.setTransform(translation);
		fakeTG.addChild(fake.createColoredSphere());
		fakeTG2.addChild(fakeTG);
		fakeBG.addChild(fakeTG2);
		if(this.refMainPanel.isPrimary)
			this.refMainPanel.secondaryBG.addChild(fakeBG);
		else
			this.refMainPanel.primaryBG.addChild(fakeBG);
		return fakeTG;
	}

	private Transform3D computeOrientation(Vector3f finalPos) {

		// Define end points as new vectors
		Vector3f fromDir = new Vector3f(1f, 0f, 0f);
		Vector3f toDir = new Vector3f(finalPos);
		toDir.normalize();

		Transform3D resultantTransform = new Transform3D();
		Vector3f translationVector = new Vector3f(finalPos);
		resultantTransform.setTranslation(translationVector);

		if (toDir.equals(fromDir) || toDir.equals(new Vector3f(0f, -1f, 0f))) {
			return resultantTransform;
		}

		// Compute the value of the angle between them (no spatial orientation
		// info though)
		float rotInRadians = fromDir.angle(toDir);

		// Compute an orthagonal vector to later define the rotation axis

		Vector3f orthagonalAxis = new Vector3f();
		orthagonalAxis.cross(fromDir, toDir);
		orthagonalAxis.normalize();

		// Create new rotation matrix
		AxisAngle4f rotAxis = new AxisAngle4f(orthagonalAxis, rotInRadians);

		resultantTransform.setRotation(rotAxis);

		return resultantTransform;
	}

//OVERRIDING SCENE3D BASIC BEHAVIOR ADDER
	protected void addBehaviors() {	
		
		//Create behaviours and add it to the scene

		//a bounding sphere specifies a region a behavior is active
		BoundingSphere bounds = new BoundingSphere(new Point3d(0.0, 0.0, 0.0),
				Double.POSITIVE_INFINITY);
		
		this.behaviorROT = new MouseRotate();
		this.behaviorROT.setTransformGroup(this.tgROT);
		this.behaviorROT.setSchedulingBounds(bounds);
		this.tgROT.addChild(behaviorROT);
		
		this.behaviorZOOM = new MouseWheelZoom();
		this.behaviorZOOM.setTransformGroup(this.tgROT);
		this.behaviorZOOM.setSchedulingBounds(bounds);
		this.tgROT.addChild(behaviorZOOM);
		
		this.behaviorTRAS = new MyMouseTranslate();
		behaviorTRAS.setTransformGroup(this.tgROT);
		behaviorTRAS.setSchedulingBounds(bounds);
		this.tgROT.addChild(behaviorTRAS);
		
		this.behaviorCURS = new MyMouseCursorSelector(this.refMainPanel);
		behaviorCURS.setTransformGroup(this.tgROT);
		behaviorCURS.setSchedulingBounds(bounds);
		this.tgROT.addChild(behaviorCURS);
		
		this.tgROT.addChild(tgSCENE);
		//Adding the scene to the main scene bg NOT HERE but in fadeIN in order to avoid glitches 
		//this.bgSCENE.addChild(tgROT);
		this.simplePick=new MyPickBehaviour(this.refMainPanel.myCanvas3D,this); 		
		this.bgSCENE.addChild(simplePick);
	}


}
