package it.polito.elite.visualization.ontoviewer.userInterfaces;

import it.polito.elite.visualization.ontoviewer.scenes3D.SceneFocusRoots1;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.net.URL;
import java.util.HashSet;
import java.util.Set;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.DefaultListModel;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTabbedPane;
import javax.swing.JTextArea;

public class LogicViewPanel extends JPanel {
	/**
	 * Comment for <code>serialVersionUID</code>
	 */
	private static final long serialVersionUID = -1003476137325736416L;
	private Pane3DScene viewerPanel;
	private JTextArea queryText;
	private DefaultListModel resList;
	private JButton execButton;
	private String savedLayout;
	public JButton saveCurrentView;
	
	public LogicViewPanel(Pane3DScene viewer) {
		// take a reference to the plugin
		this.viewerPanel = viewer;

		// setting the Panel Layout
		this.setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
		// this.setBorder(BorderFactory.createLineBorder(Color.black));

		/***********************************************************************
		 * The view management folder title *
		 **********************************************************************/
		// adding the title bar to the query panel
		this.add(this.createHeading("LOGIC VIEWS - MANAGEMENT"),
				BorderLayout.NORTH);

		/***********************************************************************
		 * The folder contents *
		 **********************************************************************/
		// Creating the panel object
		final JPanel contentPanel = new JPanel();

		// setting a GridBagLayout for the control panel
		contentPanel.setLayout(new BoxLayout(contentPanel, BoxLayout.Y_AXIS));

		// Adding a vertical space from the top
		contentPanel.add(Box.createVerticalStrut(1));
		this.populateContentPane(contentPanel);

		this.add(contentPanel, BorderLayout.CENTER);

		JTabbedPane logicViewsComposer = new JTabbedPane();

//		QueryPanel panelQuery = new QueryPanel(this.viewerPanel);
//
//		URL url = this.getClass().getClassLoader().getResource("resources/SPARQLlogicViews.gif");
//		ImageIcon iconQuery = new ImageIcon(url);
//		logicViewsComposer
//				.addTab("SPARQL Query", iconQuery, panelQuery,
//						"Selects concepts through SPARQL queries (by hand or by wizards)");

		SimpleSelectionTreePanel panelTree = new SimpleSelectionTreePanel(
				this.viewerPanel);
		URL url = this.getClass().getClassLoader().getResource("resources/classes.gif");
		ImageIcon iconTree = new ImageIcon(url);
		logicViewsComposer
				.addTab("Boolean Selection Tree", iconTree, panelTree,
						"Selects concepts through a simple boolean selection tree viewer");

		this.add(logicViewsComposer, BorderLayout.SOUTH);
	}

	private void populateContentPane(final JPanel contentPanel) {
		// creates a combo box with an inpout for each logic view defined in the
		// ontology model
		Set<String> lista = this.viewerPanel.model.getViews();
		String[] viste = new String[1];
		viste = lista.toArray(viste);
		final JComboBox savedViews = new JComboBox(viste);
		// adding the query input panel

		JPanel saveAndRoot = new JPanel(new FlowLayout(FlowLayout.CENTER));
		// Save current View Button
		saveCurrentView = new JButton("Save Current View");
		saveCurrentView.setToolTipText("In order to save a view you should be in the main \"earthlike\" scene!!");
		URL url = this.getClass().getClassLoader().getResource("resources/save.gif");
		saveCurrentView.setIcon(new ImageIcon(url));
		final String initialValue = this.viewerPanel.currentLogicViewName;
		saveCurrentView.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if(viewerPanel.scene.getSceneID()==1){
				String viewName = (String) JOptionPane.showInputDialog(
						contentPanel, "Choose a name..",
						"SAVE CURRENT LOGIC VIEW AS..",
						JOptionPane.PLAIN_MESSAGE, null, null, initialValue);
				saveLogicView(viewName, viewerPanel.conceptsOfCurrentLogicView);
				savedViews.addItem(viewName);
				savedViews.setSelectedItem(viewName);
				}
				// salva
			}
		});


		saveAndRoot.add(saveCurrentView);

		// Load Roots
		JButton loadRoots = new JButton("Load Roots");
		url = this.getClass().getClassLoader().getResource("resources/loadRoots.gif");
		loadRoots.setIcon(new ImageIcon(url));
		loadRoots.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				Set<String> roots = viewerPanel.model.getRoots();
				viewerPanel.currentLogicViewName = null;
				SceneFocusRoots1 myScene = new SceneFocusRoots1(viewerPanel,
						roots);
				viewerPanel.changeScene(myScene);
			}
		});
		saveAndRoot.add(loadRoots);

		contentPanel.add(saveAndRoot);

		// Load/Delete an existing view
		JPanel viewSelectionPane = new JPanel(new FlowLayout(FlowLayout.CENTER));

		JLabel existingViews = new JLabel("Stored Views -> ");
		existingViews.setFont(new Font("Arial", Font.BOLD, 13));
		viewSelectionPane.add(existingViews);

		if (this.viewerPanel.currentLogicViewName != null)
			savedViews.setSelectedItem(this.viewerPanel.currentLogicViewName);
		else
			savedViews.setSelectedIndex(0);
		viewSelectionPane.add(savedViews);

		JButton load = new JButton("Load ");
		url = this.getClass().getClassLoader().getResource("resources/loadScene.gif");
		load.setIcon(new ImageIcon(url));

		load.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				String viewName = (String) savedViews.getSelectedItem();
				Set<String> concepts = viewerPanel.model
						.getViewElements(viewName);
				viewerPanel.currentLogicViewName = viewName;
				SceneFocusRoots1 myScene = new SceneFocusRoots1(viewerPanel,
						concepts);
				viewerPanel.changeScene(myScene);
			}
		});
		viewSelectionPane.add(load);

		JButton delete = new JButton(" Delete ");

		url = this.getClass().getClassLoader().getResource("resources/delScene.gif");
		delete.setIcon(new ImageIcon(url));

		delete.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				String viewName = (String) savedViews.getSelectedItem();
				deleteLogicView(viewName);
				savedViews.removeItem(viewName);
			}
		});
		viewSelectionPane.add(delete);

		contentPanel.add(viewSelectionPane);
	}

	/**
	 * This method allows the creation of the title bar for the query tab of the
	 * ontosphere3D control interface. It returns the title bar as a JPanel.
	 * 
	 * @return the title bar as a JPanel
	 */
	private JPanel createHeading(String title) {
		// setting the title
		JLabel panelTitle = new JLabel(title);

		// setting the font
		Font labelFont = new Font("SansSerif", Font.BOLD, 12);
		panelTitle.setFont(labelFont);

		// setting the foreground color
		panelTitle.setForeground(Color.WHITE);

		// loading the panel icon (a circle with a question mark inside)

		URL url = this.getClass().getClassLoader()
				.getResource("resources/logicViews.gif");
		JLabel panelIcon = new JLabel(new ImageIcon(url));

		// creating the container for the title bar
		JPanel titleContainer = new JPanel();

		// setting the container layout
		titleContainer.setLayout(new FlowLayout());

		// setting the container background color
		titleContainer.setBackground(Color.LIGHT_GRAY);

		// adding the title bar components
		titleContainer.add(panelIcon);
		titleContainer.add(panelTitle);

		return titleContainer;
	}

	private void deleteLogicView(String viewName) {
		this.viewerPanel.model.deleteView(viewName);
	}

	final private void saveLogicView(String viewName, HashSet<String> set) {
		this.viewerPanel.model.saveViewElements(viewName, set);
		this.viewerPanel.currentLogicViewName = new String(viewName);
		this.viewerPanel.scene.addSceneLabel(viewName + " view");
	}
}
