package it.polito.elite.visualization.ontoviewer.userInterfaces;

import it.polito.elite.visualization.ontoviewer.models.KnowledgeBaseModel;
import it.polito.elite.visualization.ontoviewer.scenes3D.Scene3D;
import it.polito.elite.visualization.ontoviewer.scenes3D.SceneFocusRoots1;

import java.awt.BorderLayout;
import java.awt.GraphicsConfiguration;
import java.util.HashSet;

import javax.media.j3d.Background;
import javax.media.j3d.BoundingSphere;
import javax.media.j3d.BranchGroup;
import javax.media.j3d.Canvas3D;
import javax.media.j3d.DirectionalLight;
import javax.swing.JPanel;
import javax.vecmath.Color3f;
import javax.vecmath.Point3d;
import javax.vecmath.Vector3f;

import com.sun.j3d.utils.universe.SimpleUniverse;

import edu.stanford.smi.protege.model.KnowledgeBase;

/**
 * Pane3DScene extends a JPanel, its purpose consists in visualizing a Canvas3D
 * containing an interactive 3D scene used to present ontological structure
 * data. Each class present in scene3D package selects and composes element from
 * the ontology to the scene. Pane3DScene creates the Canvas3D and the 3D
 * environment, and it also manages the attacching and detaching of the scenes
 * by its changeScene method
 * 
 * @author alessio.bosca@polito.it
 * 
 */
public class Pane3DScene extends JPanel {
	private static final long serialVersionUID = 1L;

	public Scene3D scene;

	public KnowledgeBaseModel model = null;

	public BranchGroup bgROOT = null , primaryBG = null, secondaryBG = null;

	public HashSet<String> conceptsOfCurrentLogicView = null;
	public String currentLogicViewName = null;
	
	public Canvas3D myCanvas3D;

	private SimpleUniverse universe;
	
	public boolean isPrimary=false;
	
	private int oldSceneID=-1;
	
	// the protege knowledge base that is needed for building the 3D
	// representation
	private KnowledgeBase kb;

	public Pane3DScene(KnowledgeBase kb) {
		super();
		this.kb = kb;
		setBorder(javax.swing.BorderFactory.createLineBorder(	java.awt.Color.black, 1));
		setLayout(new BorderLayout());
		this.createCanvas3D();
	}

	/**
	 * Creates the Canvas3D
	 * 
	 */
	private void createCanvas3D() {
		// creo il canvas 3D
		GraphicsConfiguration config = SimpleUniverse
				.getPreferredConfiguration();
		myCanvas3D = new Canvas3D(config);
		this.add(myCanvas3D, java.awt.BorderLayout.CENTER);

		universe = new SimpleUniverse(myCanvas3D);

		bgROOT = this.createBranchGroup();
		bgROOT.addChild(createBackground());
		// lights
		bgROOT.addChild(setLight(-1.0f, -2.5f, -5.0f));
		bgROOT.addChild(setLight(3.0f, -2.5f, -3.0f));
		//Scene BGs
		primaryBG = this.createBranchGroup();
		secondaryBG = this.createBranchGroup();
		
		this.bgROOT.addChild(primaryBG);
		this.bgROOT.addChild(secondaryBG);
		
		this.getOntoFromFile();

		SceneFocusRoots1 myScene = new SceneFocusRoots1(this, null);
		this.changeScene(myScene);


		universe.addBranchGraph(bgROOT);
		universe.getViewingPlatform().setNominalViewingTransform();
	}

	public String getConceptURI() {
		return this.scene.getCurrentConceptURI();
	}

	public String getInstanceURI() {
		return this.scene.getCurrentInstanceURI();
	}

	public void changeScene(Scene3D newScene) {
		if(this.scene!=null)
			oldSceneID=this.scene.getSceneID();
		System.out.println("CAMBIO SCENA IN MAINFRAME "+oldSceneID+"->"+ newScene.getSceneID());
		this.scene = newScene;
		BranchGroup sceneBG = this.scene.getSceneBranchGroup();
		// compile and add the scene branchgroup
		sceneBG.compile();
		if(VisualizationOptions.sG_useAnimations){
			this.scene.fadeInAnimation();
			if(this.isPrimary){
				primaryBG.removeAllChildren();
				secondaryBG.addChild(sceneBG);
				isPrimary=false;
			}
			else{
				secondaryBG.removeAllChildren();
				primaryBG.addChild(sceneBG);
				isPrimary=true;
			}

		}
		else{
			primaryBG.removeAllChildren();
			primaryBG.addChild(sceneBG);
		}

	}
	

	public void redraw() {
		this.scene.recreateSceneFromScratch();
		BranchGroup sceneBG = this.scene.getSceneBranchGroup();
		primaryBG.removeAllChildren();
		primaryBG.addChild(sceneBG);
	}

	/**
	 * Loads data from the ontology
	 * 
	 */
	private void getOntoFromFile() {
		try {
			this.model = new KnowledgeBaseModel(this.kb);
		} catch (Exception e) {
			e.printStackTrace();
			
		}

	}

	/**
	 * Creates a proper BranchGroup
	 * 
	 * @return a new BranchGroup
	 */
	protected BranchGroup createBranchGroup() {
		BranchGroup bgx = new BranchGroup();
		bgx.setCapability(BranchGroup.ALLOW_DETACH);
		bgx.setCapability(BranchGroup.ALLOW_CHILDREN_READ);
		bgx.setCapability(BranchGroup.ALLOW_CHILDREN_WRITE);
		bgx.setCapability(BranchGroup.ALLOW_CHILDREN_EXTEND);
		return bgx;
	}


	private Background createBackground() {
		Background bg = new Background(VisualizationOptions.sG_background);
		bg.setApplicationBounds(new BoundingSphere(new Point3d(.0f, .0f, .0f),
				1.0));
		return bg;
	}

	private DirectionalLight setLight(float x, float y, float z) {

		DirectionalLight newLight = new DirectionalLight(new Color3f(1.0f,
				1.0f, 1.0f), new Vector3f(x, y, z));
		BoundingSphere lightBounds = new BoundingSphere(new Point3d(0.0, 0.0,
				0.0), Double.POSITIVE_INFINITY);
		newLight.setInfluencingBounds(lightBounds);
		return newLight;
	}

	public int getOldSceneID() {
		return oldSceneID;
	}

}
