package it.polito.elite.visualization.ontoviewer.userInterfaces;

import it.polito.elite.listeners.AddToViewListener;
import it.polito.elite.listeners.ListUIEnabler;
import it.polito.elite.listeners.RemoveFromViewListener;
import it.polito.elite.visualization.ontoviewer.models.ConceptData;

import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.net.URL;

import javax.swing.DefaultListModel;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTree;
import javax.swing.tree.TreePath;

public class SimpleSelectionTreePanel extends JPanel {
	
	/**
     * Comment for <code>serialVersionUID</code>
     */
    private static final long serialVersionUID = -1003476137325736416L;
	private Pane3DScene viewerPanel;
	private JScrollPane treeviewer;
	private DefaultListModel resList;
	private JTree tree;
	
	public SimpleSelectionTreePanel (Pane3DScene viewer)
	{
		//take a reference to the plugin
		this.viewerPanel = viewer;
		this.resList = new DefaultListModel();
		ListUIEnabler listUIListener = new ListUIEnabler(this.resList);
		this.resList.addListDataListener(listUIListener);
		
//		setting the Panel Layout
		this.setLayout(new BorderLayout());
		
		treeviewer=this.createSimpleTreeViewer();
		this.add(treeviewer, BorderLayout.CENTER);
		
		JPanel buttonPane = new JPanel();
		buttonPane.setLayout(new FlowLayout());
		
		JButton addButton = new JButton("Add to view");

		URL url =this.getClass().getClassLoader().getResource("resources/add_class.gif");
		addButton.setIcon(new ImageIcon(url));
		addButton.setEnabled(false);
		listUIListener.add(addButton);
		addButton.addActionListener(new AddToViewListener(this, this.resList, this.viewerPanel));
		JButton removeButton = new JButton("Remove from view");
		 url =this.getClass().getClassLoader().getResource("resources/del_class.gif");
		removeButton.setIcon(new ImageIcon(url));
		removeButton.setEnabled(false);
		listUIListener.add(removeButton);
		removeButton.addActionListener(new RemoveFromViewListener(this, this.resList, this.viewerPanel));
		buttonPane.add(removeButton);
		buttonPane.add(addButton);
		
		this.add(buttonPane, BorderLayout.SOUTH);
	}
	
	private JScrollPane createSimpleTreeViewer(){
		tree=new JTree(this.viewerPanel.model.getTree().getModel());
		
// expand paths to members of layout
		for(String uri: this.viewerPanel.conceptsOfCurrentLogicView){
			ConceptData cd=this.viewerPanel.model.getConceptData(uri);
			if(cd.getNode().getLevel()>=2)
				cd=this.viewerPanel.model.getConceptData(cd.getParent());
			TreePath path=new TreePath((cd.getNode().getPath()));
			tree.expandPath(path);
		}
		//Setting height of rows and some appearance settings
		tree.setRowHeight(30);
		tree.setRootVisible(false);
		tree.setShowsRootHandles(true);
		LayoutEditor_TreeCellRenderer renderer=new LayoutEditor_TreeCellRenderer(this.resList, this.viewerPanel, this.tree);
		tree.setCellRenderer(renderer);
		tree.setEditable(false);
		JScrollPane pane = new JScrollPane(tree);
		
		return pane;
	}
	
	public void redrawTree(){
		this.tree.clearSelection();
		this.treeviewer.validate();
		this.treeviewer.repaint();
	}
}

